@@ -1,3 +1,11 @@
+# (C) Copyright 2007-2019 Enthought, Inc., Austin, TX
+# All rights reserved.
+#
+# This software is provided without warranty under the terms of the BSD
+# license included in LICENSE.txt and may be redistributed only
+# under the conditions described in the aforementioned license.  The license
+# is also available online at http://www.enthought.com/licenses/BSD.txt
+# Thanks for using Enthought open source!
 # Standard library imports.
 import logging
 import os.path
@@ -16,6 +24,9 @@ from envisage._compat import pickle, STRING_BASE_CLASS
 # Logging.
 logger = logging.getLogger(__name__)
 
+#: Default filename for saving layout information
+DEFAULT_STATE_FILENAME = "application_memento"
+
 
 class TasksApplication(Application):
     """The entry point for an Envisage Tasks application.
@@ -29,12 +40,18 @@ class TasksApplication(Application):
     TASK_FACTORIES = 'envisage.ui.tasks.tasks'
     TASK_EXTENSIONS = 'envisage.ui.tasks.task_extensions'
 
+    # Pickle protocol to use for persisting layout information. Subclasses may
+    # want to increase this, depending on their compatibility needs. Protocol
+    # version 2 is safe for Python >= 2.3. Protocol version 4 is safe for
+    # Python >= 3.4.
+    layout_save_protocol = Int(2)
+
     #### 'TasksApplication' interface #########################################
 
     # The active task window (the last one to get focus).
     active_window = Instance('envisage.ui.tasks.task_window.TaskWindow')
 
-    # The PyFace GUI for the application.
+    # The Pyface GUI for the application.
     gui = Instance('pyface.gui.GUI')
 
     # Icon for the whole application. Will be used to override all taskWindows
@@ -52,6 +69,10 @@ class TasksApplication(Application):
     # information.
     state_location = Directory
 
+    # The filename that the application uses to persist window layout
+    # information.
+    state_filename = Unicode(DEFAULT_STATE_FILENAME)
+
     # Contributed task factories. This attribute is primarily for run-time
     # inspection; to instantiate a task, use the 'create_task' method.
     task_factories = ExtensionPoint(id=TASK_FACTORIES)
@@ -73,7 +94,7 @@ class TasksApplication(Application):
         Instance('pyface.tasks.task_window_layout.TaskWindowLayout'))
 
     # Whether to always apply the default *application level* layout when the
-    # application is started. Even if this is False, the layout state of
+    # application is started. Even if this is True, the layout state of
     # individual tasks will be restored.
     always_use_default_layout = Bool(False)
 
@@ -126,9 +147,11 @@ class TasksApplication(Application):
     def run(self):
         """ Run the application.
 
-        Returns:
-        --------
-        Whether the application started successfully (i.e., without a veto).
+        Returns
+        -------
+        bool
+            Whether the application started successfully (i.e., without a
+            veto).
         """
         # Make sure the GUI has been created (so that, if required, the splash
         # screen is shown).
@@ -152,9 +175,10 @@ class TasksApplication(Application):
     def create_task(self, id):
         """ Creates the Task with the specified ID.
 
-        Returns:
-        --------
-        The new Task, or None if there is not a suitable TaskFactory.
+        Returns
+        -------
+        pyface.tasks.task.Task
+            The new Task, or None if there is not a suitable TaskFactory.
         """
         # Get the factory for the task.
         factory = self._get_task_factory(id)
@@ -171,8 +195,8 @@ class TasksApplication(Application):
     def create_window(self, layout=None, restore=True, **traits):
         """Creates a new TaskWindow, possibly with some Tasks.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         layout : TaskWindowLayout, optional
              The layout to use for the window. The tasks described in
              the layout will be created and added to the window
@@ -188,9 +212,10 @@ class TasksApplication(Application):
              Additional parameters to pass to ``window_factory()``
              when creating the TaskWindow.
 
-        Returns:
-        --------
-        The new TaskWindow.
+        Returns
+        -------
+        envisage.ui.tasks.task_window.TaskWindow
+            The new TaskWindow.
 
         """
         from .task_window_event import TaskWindowEvent
@@ -241,17 +266,18 @@ class TasksApplication(Application):
         her window manager. It is only called via the File->Exit menu
         item. It can also, of course, be called programatically.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         force : bool, optional (default False)
             If set, windows will receive no closing events and will be
             destroyed unconditionally. This can be useful for reliably
             tearing down regression tests, but should be used with
             caution.
 
-        Returns:
-        --------
-        A boolean indicating whether the application exited.
+        Returns
+        -------
+        bool
+            A boolean indicating whether the application exited.
 
         """
         self._explicit_exit = True
@@ -326,20 +352,28 @@ class TasksApplication(Application):
         """ Loads saved application state, if possible.
         """
         state = TasksApplicationState()
-        filename = os.path.join(self.state_location, 'application_memento')
+        filename = os.path.join(self.state_location, self.state_filename)
         if os.path.exists(filename):
             # Attempt to unpickle the saved application state.
+            logger.debug('Loading application state from %s', filename)
             try:
                 with open(filename, 'rb') as f:
                     restored_state = pickle.load(f)
+            except Exception:
+                # If anything goes wrong, log the error and continue.
+                logger.exception('Error while restoring application state')
+            else:
                 if state.version == restored_state.version:
                     state = restored_state
+                    logger.debug('Application state successfully restored')
                 else:
-                    logger.warn('Discarding outdated application layout')
-            except:
-                # If anything goes wrong, log the error and continue.
-                logger.exception('Restoring application layout from %s',
-                                 filename)
+                    logger.warning(
+                        'Discarding outdated application state: '
+                        'expected version %s, got version %s',
+                        state.version, restored_state.version)
+        else:
+            logger.debug("No saved application state found at %s", filename)
+
         self._state = state
 
     def _restore_layout_from_state(self, layout):
@@ -373,13 +407,16 @@ class TasksApplication(Application):
         self._state.previous_window_layouts = window_layouts
 
         # Attempt to pickle the application state.
-        filename = os.path.join(self.state_location, 'application_memento')
+        filename = os.path.join(self.state_location, self.state_filename)
+        logger.debug('Saving application state to %s', filename)
         try:
             with open(filename, 'wb') as f:
-                pickle.dump(self._state, f)
-        except:
+                pickle.dump(self._state, f, protocol=self.layout_save_protocol)
+        except Exception:
             # If anything goes wrong, log the error and continue.
-            logger.exception('Saving application layout')
+            logger.exception('Error while saving application state')
+        else:
+            logger.debug('Application state successfully saved')
 
     #### Trait initializers ###################################################
 
